/*
  StepperL293.cpp - Stepper Library to use with L293 chip.
  Created by Martin Nawrath and Andreas Muxel, 2008.
  http://interface.khm.de
  Released into the public domain.
*/

#include "WProgram.h"
#include "StepperL293.h"

// for setting registers
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

// constructor
StepperL293::StepperL293(){};
// attach
void StepperL293::attach(int pin1, int pin2, int pin3, int pin4, int numberOfSteps)
{
	// Timer 1 prescaler to 1
	cbi(TCCR1B, CS12);
	cbi(TCCR1B, CS11);
	sbi(TCCR1B, CS10);
	// motor pins
	_motorPin1 = pin1;
	_motorPin2 = pin2;
	_motorPin3 = pin3;
	_motorPin4 = pin4;
	pinMode(_motorPin1, OUTPUT);
	pinMode(_motorPin2, OUTPUT);
	pinMode(_motorPin3, OUTPUT);
	pinMode(_motorPin4, OUTPUT);
	// PWM pins
	pinMode(9, OUTPUT);
	pinMode(10, OUTPUT);
	// init variables
	_stepperMode = 0;
	_lastTimeSpeed = 0;
	_directionStepper = 0;
	_localStep = 0;
	_currentStep = 0;
	_referenceStep = 0;
	_delaySpeed = 0;
	_numberOfSteps = numberOfSteps;
	_stepReadyStatus = 0;
}
// set mode (0 = half step mode, 1 = full step mode)
void StepperL293::setMode(int mode){
	// mode
	_stepperMode = mode;
}
// set power
void StepperL293::setPower(int power){
	// remap
	power = map(power, 0, 100, 0, 245);
	// set pwm values
	_pwmValue = power;
	_pwm23Value = _pwmValue*200;
	_pwm23Value = _pwm23Value/300;
}
// set speed
void StepperL293::setSpeed(long speed){
	// remap value
	if(_stepperMode == 0){
		speed = map(speed, 1, 100, 20, 400);
	}else{
		speed = map(speed, 1, 100, 25, 300);
	}
	// set delay
	_delaySpeed = 60L * 1000L / _numberOfSteps / speed;
}
// set step
void StepperL293::setStep(int step){
	_referenceStep = step;
}
// get step
int StepperL293::getStep(void){
	return _currentStep;
}
// get status step ready
int StepperL293::stepReady(void){
	return _stepReadyStatus;
}
// drive
void StepperL293::drive(){
	// check difference to reference step
	_differenceSteps = _referenceStep - _currentStep;
	// just if there is a difference
	if(_differenceSteps != 0){
		// set status step ready
		_stepReadyStatus = 0;
		// refresh just if delay has passed
		if (millis() - _lastTimeSpeed >= _delaySpeed) {
			// get the time
			_lastTimeSpeed = millis();
			// check difference
			if(_differenceSteps > 0){
				// set direction
				_directionStepper = 1;
			  }
			  if(_differenceSteps < 0){
				// set direction
				_directionStepper = 0;
			  }
			  // check direction
			  if (_directionStepper == 0){
				_localStep--;
				_currentStep--;
			  }
			  else {
				_localStep++;
				_currentStep++;
			  }
			  // check if half step or full step
			  if(_stepperMode == 0){
				// limes
				if (_localStep > 8){
				  _localStep = 1;
				}
				if (_localStep < 1){  
				  _localStep = 8;
				}
				// check which phase
				switch (_localStep) {
				case 1:
				//PORTD = 64;
				  digitalWrite(_motorPin1, LOW);
				  digitalWrite(_motorPin2, LOW);
				  digitalWrite(_motorPin3, LOW);
				  digitalWrite(_motorPin4, HIGH);
				  _pwm1A = _pwmValue;
				  _pwm1B = _pwmValue;
				  break;
				case 2:
				//PORTD = 80;
				  digitalWrite(_motorPin1, LOW);
				  digitalWrite(_motorPin2, HIGH);
				  digitalWrite(_motorPin3, LOW);
				  digitalWrite(_motorPin4, HIGH);
				  _pwm1A = _pwm23Value;
				  _pwm1B = _pwm23Value;
				  break;
				case 3:
				//PORTD = 16;
				  digitalWrite(_motorPin1, LOW);
				  digitalWrite(_motorPin2, HIGH);
				  digitalWrite(_motorPin3, LOW);
				  digitalWrite(_motorPin4, LOW);
				  _pwm1A = _pwmValue;
				  _pwm1B = _pwmValue;
				  break;
				case 4:
				//PORTD = 48;
				  digitalWrite(_motorPin1, LOW);
				  digitalWrite(_motorPin2, HIGH);
				  digitalWrite(_motorPin3, HIGH);
				  digitalWrite(_motorPin4, LOW);
				  _pwm1A = _pwm23Value;
				  _pwm1B = _pwm23Value;
				  break;
				case 5:
				//PORTD = 32;
				  digitalWrite(_motorPin1, LOW);
				  digitalWrite(_motorPin2, LOW);
				  digitalWrite(_motorPin3, HIGH);
				  digitalWrite(_motorPin4, LOW);
				  _pwm1A = _pwmValue;
				  _pwm1B = _pwmValue;
				  break;
				case 6:
				//PORTD = 40;
				  digitalWrite(_motorPin1, HIGH);
				  digitalWrite(_motorPin2, LOW);
				  digitalWrite(_motorPin3, HIGH);
				  digitalWrite(_motorPin4, LOW);
				  _pwm1A = _pwm23Value;
				  _pwm1B = _pwm23Value;
				  break;
				case 7:
				//PORTD = 8;
				  digitalWrite(_motorPin1, HIGH);
				  digitalWrite(_motorPin2, LOW);
				  digitalWrite(_motorPin3, LOW);
				  digitalWrite(_motorPin4, LOW);
				  _pwm1A = _pwmValue;
				  _pwm1B = _pwmValue;
				  break;
				case 8:
				//PORTD = 72;
				  digitalWrite(_motorPin1, HIGH);
				  digitalWrite(_motorPin2, LOW);
				  digitalWrite(_motorPin3, LOW);
				  digitalWrite(_motorPin4, HIGH);
				  _pwm1A = _pwm23Value;
				  _pwm1B = _pwm23Value;
				  break;
				}
			}else{
				// limes
				if (_localStep > 4){
				  _localStep = 1;
				}
				if (_localStep < 1){  
				  _localStep = 4;
				}
				// set pwm
				_pwm1A = _pwmValue;
				_pwm1B = _pwmValue;
				// check which phase
				switch (_localStep) {
				case 1:
				  digitalWrite(_motorPin1, LOW);
				  digitalWrite(_motorPin2, LOW);
				  digitalWrite(_motorPin3, LOW);
				  digitalWrite(_motorPin4, HIGH);
				  break;
				case 2:
				  digitalWrite(_motorPin1, LOW);
				  digitalWrite(_motorPin2, HIGH);
				  digitalWrite(_motorPin3, LOW);
				  digitalWrite(_motorPin4, LOW);
				  break;
				case 3:
				  digitalWrite(_motorPin1, LOW);
				  digitalWrite(_motorPin2, LOW);
				  digitalWrite(_motorPin3, HIGH);
				  digitalWrite(_motorPin4, LOW);
				  break;
				case 4:
				  digitalWrite(_motorPin1, HIGH);
				  digitalWrite(_motorPin2, LOW);
				  digitalWrite(_motorPin3, LOW);
				  digitalWrite(_motorPin4, LOW);
				  break;
				}
			}
			//
			// set pwms for motor (_pwm1A and _pwm1B)
			//
			// connect pwm to pin on timer 1, channel A
			sbi(TCCR1A, COM1A1);
			// set pwm duty
			OCR1A = _pwm1A;
			// connect pwm to pin on timer 1, channel B
			sbi(TCCR1A, COM1B1);
			// set pwm duty
			OCR1B = _pwm1B;
		}
	}else{
		// set status step ready
		_stepReadyStatus = 1;
		//
		// set pwms for motor when not moving (_pwm23Value)
		//
		// connect pwm to pin on timer 1, channel A
		sbi(TCCR1A, COM1A1);
		// set pwm duty
		OCR1A = _pwm23Value;
		// connect pwm to pin on timer 1, channel B
		sbi(TCCR1A, COM1B1);
		// set pwm duty
		OCR1B = _pwm23Value;
	}
}


// debugger
int StepperL293::debugger(void){
	return _pwm23Value;
}
